@@ -25,6 +25,7 @@ import atexit
 import binascii
 import os
 import re
+import select
 import shutil
 import signal
 import socket
@@ -54,6 +55,12 @@ vmconfig = {
     "ram": "512",
     "fw_code": "/usr/share/AAVMF/AAVMF_CODE.ms.fd",
     "fw_vars_template": "/usr/share/AAVMF/AAVMF_VARS.ms.fd"
+  },
+  (Arch.PPC64EL, Firmware.IEEE1275): {
+    "cmd": "qemu-system-ppc64el",
+    "machine": "pseries",
+    "cpu": "power10",
+    "ram": "512",
   }
 }
 
@@ -98,8 +105,7 @@ class VirtualMachine:
       self.write_efivar(EFI_GLOBAL_VARIABLE_GUID, "Timeout", b"\x00\x00")
 
     self._disk_image_path = os.path.join(self._vmdir, "disk.img")
-    if image_url is not None:
-      download_file(image_url, self._disk_image_path)
+    download_file(image_url, self._disk_image_path)
 
     self._cloud_init_seed_path = os.path.join(self._vmdir, "cloud-init.seed")
     create_cloud_init_seed(self._cloud_init_seed_path)
@@ -185,11 +191,7 @@ class VirtualMachine:
       if f.tell() < vars_length:
         f.write(b"\xff" * (vars_length - f.tell()))
 
-  def start(self, ephemeral_snapshot=False, wait=True, tapname=None, tpm=None):
-    serialcon = f"unix:{self._serial_socket},server,nowait"
-    if DEBUG:
-      serialcon += ",logfile=/dev/stdout"
-
+  def start(self, ephemeral_snapshot=False, wait=True, tapname=None):
     if kvm_supported(self._arch):
       accel = "kvm"
       cpu = "host"
@@ -202,8 +204,10 @@ class VirtualMachine:
       "-M", f"{self._config['machine']},accel={accel}",
       "-cpu", cpu,
       "-nographic",
+      "-vga", "none",
       "-monitor", f"unix:{self._monitor_socket},server,nowait",
-      "-serial", serialcon,
+      "-serial", f"unix:{self._serial_socket},server,nowait",
+      "-device", "virtio-rng-pci",
       "-device", "vhost-vsock-pci,guest-cid=3",
       # First iface for network access and SSH forwarding to host
       "-netdev", "user,id=net0,hostfwd=tcp::2222-:22",
@@ -218,13 +222,6 @@ class VirtualMachine:
       # "-global", "isa-debugcon.iobase=0x402"
     ]
 
-    if tpm is not None:
-      opts += [
-        "-chardev", f"socket,id=chrtpm,path={tpm.ctrlsock()}",
-        "-tpmdev", "emulator,id=tpm0,chardev=chrtpm",
-        "-device", "tpm-tis,tpmdev=tpm0",
-      ]
-
     if self._firmware == Firmware.UEFI:
       opts += [
         "-drive", f"file={self._config['fw_code']},if=pflash,format=raw,unit=0,readonly=on",
@@ -284,9 +281,24 @@ class VirtualMachine:
       self.waitshutdown()
 
   def waitboot(self):
-    connfd, _ = self._host_vsock.accept()
-    assert connfd.recv(3) == b"RDY"
-    connfd.close()
+    # Wait for serial to come up
+    while not os.access(self._serial_socket, os.F_OK):
+      pass
+    # Wait for serial data *or* notifier connection
+    with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as serfd:
+      serfd.connect(self._serial_socket)
+      while True:
+        rlist, _, _ = select.select((serfd, self._host_vsock,), (), ())
+        if DEBUG and serfd in rlist:
+          # Dump out boot console filtering escapes
+          s = serfd.recv(4096)
+          sys.stdout.buffer.write(b"".join(bytes((c,)) for c in s if c == ord('\n') or chr(c).isprintable()))
+        if self._host_vsock in rlist:
+          connfd, _ = self._host_vsock.accept()
+          assert connfd.recv(3) == b"RDY"
+          connfd.close()
+          # No need to wait any longer
+          return
 
   def waitserial(self, data):
     # Wait for serial to come up
